"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnvironmentLive = exports.Environment = void 0;
const effect_1 = require("effect");
const Option = __importStar(require("effect/Option"));
const types_1 = require("./types");
class Environment extends effect_1.Context.Tag('Environment')() {
}
exports.Environment = Environment;
exports.EnvironmentLive = effect_1.Layer.succeed(Environment, Environment.of({
    getEnv: () => effect_1.Effect.all([
        effect_1.Config.string('JIRA_API_URL'),
        effect_1.Config.option(effect_1.Config.string('JIRA_PAT')),
        effect_1.Config.option(effect_1.Config.string('JIRA_USER_EMAIL')),
        effect_1.Config.option(effect_1.Config.string('JIRA_API_TOKEN')),
        effect_1.Config.option(effect_1.Config.string('JIRA_KEY_PREFIX')),
    ], { concurrency: 'unbounded', mode: 'validate' }).pipe(effect_1.Effect.catchAll((maybeErrors) => {
        return effect_1.Effect.fail((0, types_1.AppConfigError)({
            message: Option.reduceCompact(maybeErrors, [], (acc, e) => acc.concat(effect_1.ConfigError.reduceWithContext(e, void 0, mkCollectConfigErrorMessagesReducer()))).join('\n'),
        }));
    }), effect_1.Effect.flatMap(([jiraApiUrl, jiraPat, jiraUserEmail, jiraApiToken, defaultJiraKeyPrefix,]) => effect_1.Effect.gen(function* (_) {
        if (Option.isSome(jiraPat) &&
            (Option.isSome(jiraUserEmail) || Option.isSome(jiraApiToken))) {
            return yield* _(effect_1.Effect.fail((0, types_1.AppConfigError)({
                message: 'Please provide either JIRA_PAT (for Jira Data Center) or JIRA_API_TOKEN and JIRA_USER_EMAIL (for Jira Cloud), but not both',
            })));
        }
        if (Option.isSome(jiraPat)) {
            return {
                jiraAuth: (0, types_1.JiraDataCenterAuth)({
                    jiraPat: (0, types_1.JiraPat)(jiraPat.value),
                }),
                jiraApiUrl: (0, types_1.JiraApiUrl)(jiraApiUrl),
                defaultJiraKeyPrefix: Option.map(defaultJiraKeyPrefix, types_1.JiraKeyPrefix),
            };
        }
        if (Option.isSome(jiraApiToken) && Option.isSome(jiraUserEmail)) {
            return {
                jiraAuth: (0, types_1.JiraCloudAuth)({
                    jiraApiToken: (0, types_1.JiraApiToken)(jiraApiToken.value),
                    jiraUserEmail: (0, types_1.JiraUserEmail)(jiraUserEmail.value),
                }),
                jiraApiUrl: (0, types_1.JiraApiUrl)(jiraApiUrl),
                defaultJiraKeyPrefix: Option.map(defaultJiraKeyPrefix, types_1.JiraKeyPrefix),
            };
        }
        return yield* _(effect_1.Effect.fail((0, types_1.AppConfigError)({
            message: 'Please provide JIRA_PAT (for Jira Data Center) or JIRA_API_TOKEN and JIRA_USER_EMAIL (for Jira Cloud).',
        })));
    }))),
}));
const mkCollectConfigErrorMessagesReducer = () => {
    const collectMessage = (_, _path, message) => [message];
    const mergeErrors = (_, left, right) => [...left, ...right];
    return {
        andCase: mergeErrors,
        orCase: mergeErrors,
        invalidDataCase: collectMessage,
        missingDataCase: collectMessage,
        sourceUnavailableCase: collectMessage,
        unsupportedCase: collectMessage,
    };
};
