"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JiraClientLive = exports.JiraClient = void 0;
const effect_1 = require("effect");
const Http = __importStar(require("@effect/platform/HttpClient"));
// eslint-disable-next-line node/no-extraneous-import
const ArrayFormatter = __importStar(require("@effect/schema/ArrayFormatter"));
const environment_1 = require("./environment");
const types_1 = require("./types");
class JiraClient extends effect_1.Context.Tag('JiraClient')() {
}
exports.JiraClient = JiraClient;
exports.JiraClientLive = effect_1.Layer.effect(JiraClient, effect_1.Effect.all([environment_1.Environment, Http.client.Client]).pipe(effect_1.Effect.map(([env, httpClient]) => JiraClient.of({
    getJiraIssue: (issueId) => effect_1.Effect.gen(function* (_) {
        const endPoint = `/rest/api/latest/issue/${issueId}`;
        const { jiraAuth, jiraApiUrl } = yield* _(env.getEnv());
        return yield* _(Http.request
            .get(jiraApiUrl + endPoint, {
            headers: {
                Authorization: buildJiraAuthorizationHeader(jiraAuth),
                Accept: 'application/json',
            },
        })
            .pipe(Http.client.filterStatusOk(httpClient), effect_1.Effect.flatMap(Http.response.schemaBodyJson(types_1.JiraIssueSchema)), effect_1.Effect.scoped, handleJiraClientErrors));
    }),
}))));
const buildJiraAuthorizationHeader = (jiraAuth) => {
    switch (jiraAuth._tag) {
        case 'JiraCloudAuth':
            return `Basic ${Buffer.from(`${jiraAuth.jiraUserEmail}:${jiraAuth.jiraApiToken}`).toString('base64')}`;
        case 'JiraDataCenterAuth':
            return `Bearer ${jiraAuth.jiraPat}`;
    }
};
const handleJiraClientErrors = (0, effect_1.flow)(effect_1.Effect.catchTag('ParseError', (e) => effect_1.Effect.fail((0, types_1.JiraApiError)({
    message: [
        'Failed to parse ticket response from Jira:',
        ...ArrayFormatter.formatError(e).map((issue) => `'${issue.path.join(' ')}': '${issue.message}'`),
    ].join('\n'),
}))), effect_1.Effect.catchTag('RequestError', (e) => effect_1.Effect.fail((0, types_1.JiraApiError)({
    message: `Failed to make ticket request to Jira. Reason: ${e.reason}`,
}))), effect_1.Effect.catchTag('ResponseError', (e) => effect_1.Effect.fail((0, types_1.JiraApiError)({
    message: e.response.status === 404
        ? 'Jira returned status 404. Make sure the ticket exists.'
        : `Jira Ticket request returned failure. Reason: ${e.reason}${typeof e.error === 'string' ? ` (${e.error})` : ''} StatusCode: ${e.response.status}`,
}))));
